package simple.documentation.framework.extensions;

import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorCaretListener;
import ro.sync.ecss.extensions.api.AuthorExtensionStateListener;
import ro.sync.ecss.extensions.api.AuthorListener;
import ro.sync.ecss.extensions.api.AuthorMouseListener;
import ro.sync.ecss.extensions.api.OptionChangedEvent;
import ro.sync.ecss.extensions.api.OptionListener;
import simple.documentation.framework.filters.SDFDocumentFilter;
import simple.documentation.framework.listeners.SDFAuthorCaretListener;
import simple.documentation.framework.listeners.SDFAuthorListener;
import simple.documentation.framework.listeners.SDFAuthorMouseListener;

/**
 * Simple Document Framework state listener used to register custom listeners(caret listener, mouse 
 * listener, document listener and option listener) when the framework is activated. 
 *
 */
public class SDFAuthorExtensionStateListener implements AuthorExtensionStateListener {
  /**
   * The key used to store a custom option. 
   */
  private String customKey = "sdf.custom.option.key";

  /**
   * Custom caret listener to be added on activate and removed on deactivate
   */
  private AuthorCaretListener sdfCaretListener;

  /**
   * Custom mouse listener
   */
  private AuthorMouseListener sdfMouseListener;

  /**
   * Option listener to be added in the option storage.
   */
  private OptionListener sdfOptionListener;

  /**
   * Custom author listener 
   */
  private AuthorListener sdfAuthorDocumentListener;

  /**
   * The SDF Author extension is activated.
   */
  public void activated(final AuthorAccess authorAccess) {
    sdfOptionListener = new OptionListener(customKey) {
      @Override
      public void optionValueChanged(OptionChangedEvent newValue) {
        // The custom option changed. 
      }
    };

    // Add document filter.
    authorAccess.getDocumentController().setDocumentFilter(new SDFDocumentFilter(authorAccess));

    // Add an option listener.
    authorAccess.getOptionsStorage().addOptionListener(sdfOptionListener);

    // Add author document listeners.
    sdfAuthorDocumentListener = new SDFAuthorListener(authorAccess);
    authorAccess.getDocumentController().addAuthorListener(sdfAuthorDocumentListener);

    // Add mouse listener.
    sdfMouseListener = new SDFAuthorMouseListener(authorAccess);
    authorAccess.getEditorAccess().addAuthorMouseListener(sdfMouseListener);

    // Add caret listener.
    sdfCaretListener = new SDFAuthorCaretListener(authorAccess);
    authorAccess.getEditorAccess().addAuthorCaretListener(sdfCaretListener);

    // Other custom initializations...
    authorAccess.getEditorAccess().select(0, 100);
  }

  /**
   * The SDF Author extension is deactivated.
   */
  public void deactivated(AuthorAccess authorAccess) {

    // Remove the option listener.
    authorAccess.getOptionsStorage().removeOptionListener(sdfOptionListener);

    // Remove document listeners.
    authorAccess.getDocumentController().removeAuthorListener(sdfAuthorDocumentListener);

    // Remove mouse listener.
    authorAccess.getEditorAccess().removeAuthorMouseListener(sdfMouseListener);

    // Remove caret listener.
    authorAccess.getEditorAccess().removeAuthorCaretListener(sdfCaretListener);

    // Other actions...
  }

  public String getDescription() {
    return "Simple Document Framework state listener";
  }
}
