/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package simple.documentation.framework.operations;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.swing.text.BadLocationException;
import javax.swing.tree.TreePath;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorDocument;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.structure.AuthorOutlineCustomizer;
import simple.documentation.framework.extensions.SDFAuthorOutlineCustomizer;

/**
 * Select in the Outline the children nodes of the node at caret.
 */
public class SelectNodesInOutlinerOperation implements AuthorOperation {

  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    //  Renderer customizer used to find if a given node is filtered in the Outline.
    AuthorOutlineCustomizer rendererCustomizer = new SDFAuthorOutlineCustomizer(); 
    
     int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
     try {
       // Node at caret position
      AuthorNode currentNode = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);
      if (currentNode instanceof AuthorElement) {
        AuthorElement currentElement = (AuthorElement)currentNode;
        // Find the content nodes
        List<AuthorNode> contentNodes = currentElement.getContentNodes();
        List<TreePath> selectTreePaths = new ArrayList<TreePath>();
        LinkedList<AuthorNode> reversedPath = findReversedPath(authorAccess, rendererCustomizer,
            currentNode);
        
        for (AuthorNode authorNode : contentNodes) {
          if (!rendererCustomizer.ignoreNode(authorNode)) {
            LinkedList<AuthorNode> pathList = new LinkedList<AuthorNode>(reversedPath);
            pathList.add(authorNode);
            // Add the children tree path in the selected tree paths list
            selectTreePaths.add(new TreePath(pathList.toArray(new AuthorNode[0])));
          }
        }
        
        // Select the children tree paths in the Outline
        authorAccess.getOutlineAccess().setSelectionPaths(selectTreePaths.toArray(new TreePath[0]));
      }
     } catch (BadLocationException e) {
       e.printStackTrace();
     }
  }

  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Select nodes in the Outliner";
  }

  /**
   * Builds the list of node parents up to and including the root node, where the
   * original node is the last element in the returned array. 
   * 
   * @param authorAccess The author access.
   * @param rendererCustomizer Renderer customizer used to find if a given node 
   * is filtered in the Outline.
   * @param node The node to reverse path for.
   * @return The path nodes list.
   */
  private LinkedList<AuthorNode> findReversedPath(AuthorAccess authorAccess,
      AuthorOutlineCustomizer rendererCustomizer, AuthorNode node) {
    AuthorDocument document = authorAccess.getDocumentController().getAuthorDocumentNode();
    
    // Builds the path.
    LinkedList<AuthorNode> reversedPath = new LinkedList<AuthorNode>();
    while (node != null && !rendererCustomizer.ignoreNode(node)) {
      reversedPath.addFirst(node);
      if (node == document) {
        // Just added root.
        break;
      }
      node = node.getParent();
    }
    return reversedPath;
  }
}
