/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api;

/**
 * Base class for Author input events.
 */
public abstract class AuthorInputEvent {
  /**
   * This flag indicates that the Shift key was down when the event
   * occurred. The value is 1 << 0.
   */
  public static final int SHIFT_PRESSED = 1 << 0;

  /**
   * This flag indicates that the Control key was down when the event
   * occurred. The value is 1 << 1.
   */
  public static final int CTRL_PRESSED = 1 << 1;

  /**
   * This flag indicates that the Meta key was down when the event
   * occurred. The value is 1 << 2.
   */
  public static final int META_PRESSED = 1 << 2;

  /**
   * This flag indicates that the Alt key was down when
   * the event occurred. The value is 1 << 3.
   */
  public static final int ALT_PRESSED = 1 << 3;

  /**
   * This flag indicates that the AltGraph key was down when
   * the event occurred. The value is 1 << 5.
   */
  public static final int ALT_GRAPH_PRESSED = 1 << 5;
  
  /**
   * The state of the modifier mask at the time the input
   * event was fired.
   */
  public final int modifiers;
  
  /**
   * States whether or not the event has been consumed.
   */
  public boolean consumed;
 
  /**
   * Constructor for author input event.
   * 
   * @param modifiers The modifiers.
   */
  public AuthorInputEvent(int modifiers) {
    this.modifiers = modifiers;
  }
  
  /**
   * Set the consumed flag for the event.
   */
  public void consume() {
    this.consumed = true;
  }
  
  /**
   * @return <code>true</code> if the event was consumed.
   */
  public boolean isConsumed() {
    return consumed;
  }
  
  /**
   * @return <code>true</code> if SHIFT key was pressed.
   */
  public boolean isShiftPressed() {
    return (modifiers & SHIFT_PRESSED) != 0;
  }
  
  /**
   * @return <code>true</code> if CTRL key was pressed.
   */
  public boolean isCtrlPressed() {
    return (modifiers & CTRL_PRESSED) != 0;
  }
  
  /**
   * @return <code>true</code> if ALT key was pressed.
   */
  public boolean isAltPressed() {
    return (modifiers & ALT_PRESSED) != 0;
  }
  
  /**
   * @return <code>true</code> if ALT GR key was pressed.
   */
  public boolean isAltGraphPressed() {
    return (modifiers & ALT_GRAPH_PRESSED) != 0;
  }
  
  /**
   * @return <code>true</code> if META key was pressed.
   */
  public boolean isMetaPressed() {
    return (modifiers & META_PRESSED) != 0;
  }
  
  /**
   * @return Returns the keyboard modifiers associated with the event.
   */
  public int getModifiers() {
    return modifiers;
  }
}