/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.commons.operations;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;

/**
 * An implementation of an insert/replace operation for an argument of type {@link String}.
 */
public class InsertOrReplaceTextOperation implements AuthorOperation {
  /**
   * The name of the element.
   */
  private static final String ARGUMENT_NAME = "text";
  
  /**
   * The arguments array.
   */
  private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
    new ArgumentDescriptor(
        ARGUMENT_NAME,
        ArgumentDescriptor.TYPE_STRING,
        "The text to be inserted"),
        SCHEMA_AWARE_ARGUMENT_DESCRIPTOR
  };
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(AuthorAccess, ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args) {
    // Simple text replace.
    AuthorDocumentController controller = authorAccess.getDocumentController();
    controller.beginCompoundEdit();
    boolean deleteSelection = false;
    try {
      Object argVal = args.getArgumentValue(ARGUMENT_NAME);
      // Test if argument is a String.
      if (argVal != null && argVal instanceof String) {
        // Replace operation.
        if (authorAccess.getEditorAccess().hasSelection()) {
          // Delete old selection.
          deleteSelection = true;
          authorAccess.getEditorAccess().deleteSelection();
        }

        Object schemaAwareArgumentValue = args.getArgumentValue(SCHEMA_AWARE_ARGUMENT);
        if(AuthorConstants.ARG_VALUE_FALSE.equals(schemaAwareArgumentValue)) {
          // Insert fragment at specfied position.
          authorAccess.getDocumentController().insertText( authorAccess.getEditorAccess().getCaretOffset(), (String) argVal);
        } else {
          // Insert the text fragment schema aware.
          AuthorDocumentFragment textFragment = authorAccess.getDocumentController().createNewDocumentTextFragment((String) argVal);
          authorAccess.getDocumentController().insertFragmentSchemaAware(authorAccess.getEditorAccess().getCaretOffset(), textFragment);
        }
      } else {
        throw new IllegalArgumentException("The argument value was not defined, it is " + argVal);
      }
    } catch (AuthorOperationException e) {
      if(deleteSelection) {
        // Paste was rejected, undo selection removal
        controller.cancelCompoundEdit();
      }
    } finally {
      controller.endCompoundEdit();
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return ARGUMENTS;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getDescription()
   */
  public String getDescription() {    
    return "Insert a simple text. If selection is present, the selected text will be replaced with the new one.";
  }
}