/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.commons.table.operations.xhtml;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.commons.table.operations.TableInfo;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizerConstants.ColumnWidthsType;

/**
 * Operation used to insert a XHTML table.
 */
public class InsertTableOperation implements AuthorOperation {

	/**
	 * Namespace for XHTML v.1.x.
	 */
  private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
    throws IllegalArgumentException, AuthorOperationException {
    // Show the 'Insert table' dialog.
    TableInfo tableInfo = null;
    if (authorAccess.getWorkspaceAccess().isStandalone()) {
      tableInfo = new SAXHTMLTableCustomizerInvoker().customizeTable(authorAccess);
    } else {
      tableInfo = new ECXHTMLTableCustomizerInvoker().customizeTable(authorAccess);
    }
    if (tableInfo != null) {
      // Insert the table.
      authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
          getTableXMLFragment(tableInfo, NAMESPACE).toString(), 
          authorAccess.getEditorAccess().getCaretOffset());
    } else {
      // User canceled the operation.
    }
  }

  /**
   * Add table body.
   * 
   * @param tableXMLFragment The string buffer representing the table XML fragment.
   * The table body fragment will be added to this table fragment.
   * @param tableInfo The table info containing informations about the table rows
   * and columns number 
   */
  private static void addTableBody(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    tableXMLFragment.append("<tbody>");
    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<tr>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        tableXMLFragment.append("<td></td>"); 
      }
      tableXMLFragment.append("</tr>"); 
    }
    tableXMLFragment.append("</tbody>");
  }

  /**
   * Add table column specifications. The table will be inserted with proportional column widths.
   * 
   * @param tableXMLFragment The string buffer representing the table XML fragment.
   * The table columns specification fragment will be added to this table fragment.
   * @param tableInfo The table info containing informations about the table 
   * columns number 
   */
  private static void addTableCols(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    ColumnWidthsType columnsWidthsType = tableInfo.getColumnsWidthsType();
    if (columnsWidthsType != ColumnWidthsType.DYNAMIC_COL_WIDTHS) {
      String colWidth = null;
      if (columnsWidthsType == ColumnWidthsType.PROPORTIONAL_COL_WIDTHS) {
        // Proportional widths
        colWidth = "1.0*";
      } else {
        // Fixed widths
        colWidth = "75pt";
      }
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<col width=\"" + colWidth + "\"/>");
      }
    }
  }
  
  /**
   * Add table footer.
   * 
   * @param tableXMLFragment The string buffer representing the table XML fragment.
   * The table footer fragment will be added to this table fragment.
   * @param tableInfo The table info containing informations about the table 
   * columns number 
   */
  private static void addTableFooter(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateFooter()) {
      tableXMLFragment.append("<tfoot><tr>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<td></td>");
      }
      tableXMLFragment.append("</tr></tfoot>");
    }
  }

  /**
   * Add table header.
   * 
   * @param tableXMLFragment The string buffer representing the table XML fragment.
   * The table header fragment will be added to this table fragment.
   * @param tableInfo The table info containing informations about the table 
   * columns number 
   */
  private static void addTableHeader(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<thead><tr>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<th></th>");
      }
      tableXMLFragment.append("</tr></thead>");
    }
  }

  /**
   * No arguments. The operation will display a dialog for choosing the table attributes.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Insert a XHTML table";
  }

  /**
   * Compute the XML fragment for the table to be inserted.
   *  
   * @param tableInfo The table information.
   * @param namespace The table element namespace.
   * @return  The XML fragment for the table to be inserted.
   */
  private static StringBuffer getTableXMLFragment(TableInfo tableInfo, String namespace) {
    // Create the table XML fragment.
    StringBuffer tableXMLFragment = new StringBuffer();
    // Table element.
    // Title was specified, insert a table with title.
    tableXMLFragment.append(
        "<table xmlns=\"" + namespace + "\" " +
        "frame=\"" + tableInfo.getFrame() + "\">");
    if(tableInfo.getTitle() != null) {
      tableXMLFragment.append("<caption>" + tableInfo.getTitle() + "</caption>");
    }

    // Add table column specifications.
    addTableCols(tableXMLFragment, tableInfo);

    // Add table header.
    addTableHeader(tableXMLFragment, tableInfo);

    // Add table footer.
    addTableFooter(tableXMLFragment, tableInfo);

    // Add table body.
    addTableBody(tableXMLFragment, tableInfo);

    tableXMLFragment.append("</table>");
    
    return tableXMLFragment;
  }

}