/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita;

import java.util.List;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.contentcompletion.xml.CIAttribute;
import ro.sync.contentcompletion.xml.CIElement;
import ro.sync.contentcompletion.xml.ContextElement;
import ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter;
import ro.sync.ecss.extensions.api.AuthorSchemaManager;
import ro.sync.ecss.extensions.api.InvalidEditException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResult;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResultsImpl;

/**
 * Specific editing support for DITA documents. Handles typing and paste events inside steps, steps-unordered (a 
 * new step with a cmd will be created and the typing/paste content be put inside it) and tables (CALS and simple).
 */
public class DITASchemaAwareEditingHandler extends AuthorSchemaAwareEditingHandlerAdapter {
  /** 
   * Logger for logging. 
   */
  private static Logger logger = Logger.getLogger(DITASchemaAwareEditingHandler.class.getName());
  private static final String STEPS_CLASS = "task/steps";
  private static final String STEP_CLASS = "task/step";
  private static final String STEPS_UNORDERED_CLASS = "task/steps-unordered";
  private static final String SIMPLE_TABLE_CLASS = "topic/simpletable";
  private static final String SIMPLE_TABLE_ROW_CLASS = "topic/strow";
  private static final String SIMPLE_TABLE_ENTRY_CLASS = "topic/stentry";
  private static final String REL_TABLE_CLASS = "map/reltable";
  private static final String REL_TABLE_ROW_CLASS = "map/relrow";
  private static final String REL_TABLE_ENTRY_CLASS = "map/relcell";
  private static final String TABLE_HEAD_CLASS = "topic/thead";
  private static final String TABLE_BODY_CLASS = "topic/tbody";
  private static final String CALS_TABLE_ROW_CLASS = "topic/row";
  private static final String CALS_TABLE_ENTRY_CLASS = "topic/entry";
  private static final String CMD_CLASS = "task/cmd";
  private static final String TOPIC_CLASS = "topic/topic";
  private static final String TITLE_CLASS = "topic/title";
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter#handleTyping(int, char, ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public boolean handleTyping(int offset, char ch, AuthorAccess authorAccess) throws InvalidEditException {
    boolean handleTyping = false;
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    if (!authorSchemaManager.isLearnSchema() && 
        !authorSchemaManager.hasLoadingErrors() &&
        authorSchemaManager.getAuthorSchemaAwareOptions().isEnableSmartTyping()) {
      try {
        AuthorDocumentFragment characterFragment = 
          authorAccess.getDocumentController().createNewDocumentTextFragment(String.valueOf(ch));    
        handleTyping = handleInsertionEvent(offset, new AuthorDocumentFragment[] {characterFragment}, authorAccess);
      } catch (AuthorOperationException e) {
        throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
      }
    }
    return handleTyping;    
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter#handlePasteFragment(int, ro.sync.ecss.extensions.api.node.AuthorDocumentFragment[], int, ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public boolean handlePasteFragment(int offset, AuthorDocumentFragment[] fragmentsToInsert,
      int actionId, AuthorAccess authorAccess) throws InvalidEditException {
    boolean handleInsertionEvent = false;
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    if (!authorSchemaManager.isLearnSchema() && 
        !authorSchemaManager.hasLoadingErrors() &&
        authorSchemaManager.getAuthorSchemaAwareOptions().isEnableSmartPaste()) {
      handleInsertionEvent = handleInsertionEvent(offset, fragmentsToInsert, authorAccess);
    }
    return handleInsertionEvent;
  }
  
  /**
   * Handle an insertion event (either typing or paste).
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   * 
   * @throws InvalidEditException The event was rejected because it is invalid.
   */
  private boolean handleInsertionEvent(
      int offset, 
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess) throws InvalidEditException {
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    boolean handleEvent = false;
    try {
      // The fragment can't just be inserted at the given offset.
      AuthorNode nodeAtInsertionOffset = authorAccess.getDocumentController().getNodeAtOffset(offset);
      if (logger.isDebugEnabled()) {
        logger.debug("nodeAtInsertionOffset " + nodeAtInsertionOffset);
      }
      
      if(isElementWithClass(nodeAtInsertionOffset, TOPIC_CLASS)) {
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInTopic(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if(isElementWithClass(nodeAtInsertionOffset, STEPS_CLASS) 
          || isElementWithClass(nodeAtInsertionOffset, STEPS_UNORDERED_CLASS)) {
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInSteps(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if(isElementWithClass(nodeAtInsertionOffset, STEP_CLASS) && 
          nodeAtInsertionOffset.getStartOffset() + 1 == nodeAtInsertionOffset.getEndOffset()) {
        // Insertion in empty 'step'
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInStep(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if (isElementWithClass(nodeAtInsertionOffset, TABLE_BODY_CLASS) 
          || isElementWithClass(nodeAtInsertionOffset, TABLE_HEAD_CLASS)
          || isElementWithClass(nodeAtInsertionOffset, SIMPLE_TABLE_CLASS)
          || isElementWithClass(nodeAtInsertionOffset, REL_TABLE_CLASS)) {
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInTable(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      }
    } catch (BadLocationException e) {
      throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
    } catch (AuthorOperationException e) {
      throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
    }
    return handleEvent;    
  }
  
  /**
   * Try to handle invalid insertion events in elements with 'topic/topic' in class attribute. 
   * The fallback is to insert the <code>fragmentsToInsert</code> into a 'title' element if is possible.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInTopic(int offset,
      AuthorDocumentFragment[] fragmentsToInsert, AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager)  throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    AuthorElement topicElement = (AuthorElement) authorAccess.getDocumentController().getNodeAtOffset(offset);
    // If the topic is empty
    if (topicElement.getStartOffset() + 1 == topicElement.getEndOffset()) {
      // Typing/paste inside a topic. We will try to wrap the fragment into a new title element
      WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
      // Derive the context by adding a title
      String titleElementName = getElementName(authorSchemaManager, context, TITLE_CLASS);
      pushContextElement(context, titleElementName);
      // Test if fragments can be inserted in 'title' element
      if (authorSchemaManager.canInsertDocumentFragments(
          fragmentsToInsert, 
          context, 
          AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {

        StringBuilder xmlFragment = new StringBuilder("<").append(titleElementName);
        xmlFragment.append("/>");
        if (logger.isDebugEnabled()) {
          logger.debug("Insert " + xmlFragment);
        }
        authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

        // Get the newly inserted title element.
        AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 1);
        int insertionOffset = -1;
        for (int i = 0; i < fragmentsToInsert.length; i++) {
          if(insertionOffset == -1) {
            insertionOffset = newParaNode.getEndOffset();
          }
          authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
        }
        lastHandlerResult = new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
        ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, insertionOffset);
        handleEvent = true;
      } else {
        if (logger.isDebugEnabled()) {
          logger.debug("Fragments cannot be inserted in a para element.");
        }
      }
    }
    return handleEvent;
  }

  /**
   * Get the element name from the given context and element class.
   *  
   * @param schemaManager
   * @param context
   * @param elementClass
   * @return
   */
  private String getElementName(AuthorSchemaManager schemaManager, WhatElementsCanGoHereContext context, String elementClass) {
    String elementName = null;
    List<CIElement> childrenElements = schemaManager.getChildrenElements(context);
    for (CIElement ciElement : childrenElements) {
      List<CIAttribute> attributes = ciElement.getAttributes();
      for (CIAttribute ciAttribute : attributes) {
        if (ciAttribute.getName().equals("class")) {
          if (ciAttribute.getDefaultValue().contains(" " + elementClass + " ")) {
            elementName = ciElement.getName();
          }
          break;
        }
      }
      if (elementName != null) {
        break;
      }
    }
    return elementName;
  }

  /**
   * Try to handle invalid insertion events in 'steps' or 'steps-unordered'. The fallback is to insert 
   * the <code>fragmentsToInsert</code> into a 'steps/cmd' structure if is possible.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInSteps(int offset,
      AuthorDocumentFragment[] fragmentsToInsert, AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside an steps or ordered steps. We will try to insert a step item with a cmd inside 
    // and perform the insertion inside the cmd element.
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a step item with a cmd inside.
    String stepElementName = getElementName(authorSchemaManager, context, STEP_CLASS);
    pushContextElement(context, stepElementName);
    String cmdElementName = getElementName(authorSchemaManager, context, CMD_CLASS);
    pushContextElement(context, cmdElementName);
    // Test if fragments can be inserted in 'cmd' element
    if (authorSchemaManager.canInsertDocumentFragments(
        fragmentsToInsert, 
        context, 
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
      // Create a step/cmd structure and insert fragments inside the new cmd
      StringBuilder xmlFragment = new StringBuilder("<").append(stepElementName);
      xmlFragment.append("><").append(cmdElementName).append("/></").append(stepElementName).append(">");
      if (logger.isDebugEnabled()) {
        logger.debug("Insert " + xmlFragment);
      }
      // Insert step/cmd
      authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

      // Insert fragments
      int insertionOffset = -1;
      AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 2);            
      for (int i = 0; i < fragmentsToInsert.length; i++) { 
        if(insertionOffset == -1) {
          insertionOffset = newParaNode.getEndOffset();
        }
        authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
      }
      
      lastHandlerResult = new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
      ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, insertionOffset);      
      handleEvent = true;
    } else {
      if (logger.isDebugEnabled()) {
        logger.debug("Fragments cannot be inserted in a para element.");
      }
    }
    return handleEvent;
  }
  
  /**
   * Derive the given context by adding the given element.
   */
  private void pushContextElement(WhatElementsCanGoHereContext context, String elementName) {
    ContextElement contextElement = new ContextElement();
    contextElement.setQName(elementName);
    context.pushContextElement(contextElement, null);
  }
  
  /**
   * @return <code>true</code> if the given node is an element with the given class.
   */
  private boolean isElementWithClass(AuthorNode node, String elementClass) {
    boolean result = false;
    if(node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue classVal = element.getAttribute("class");
      if(classVal != null) {
        result = classVal.getValue().contains(" " + elementClass + " ");
      }
    }
    return result;
  }
  
  /**
   * Try to handle invalid insertion events in 'listitem'. The fallback is to insert 
   * the <code>fragmentsToInsert</code> into a 'para' element if is possible.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInStep(
      int offset,
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside a step. We will try to wrap the fragment into a new cmd element
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a list item with a cmd inside.
    String cmdElementName = getElementName(authorSchemaManager, context, CMD_CLASS);
    pushContextElement(context, cmdElementName);
    // Test if fragments can be inserted in 'cmd' element
    if (authorSchemaManager.canInsertDocumentFragments(
        fragmentsToInsert, 
        context, 
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
      
      // Insert 'cmd' in step
      StringBuilder xmlFragment = new StringBuilder("<").append(cmdElementName);
      xmlFragment.append("/>");
      if (logger.isDebugEnabled()) {
        logger.debug("Insert " + xmlFragment);
      }
      authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);
  
      // Get the newly inserted cmd element.
      int insertionOffset = -1;
      AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 1);            
      for (int i = 0; i < fragmentsToInsert.length; i++) { 
        if(insertionOffset == -1) {
          insertionOffset = newParaNode.getEndOffset();
        }
        authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
      }
      
      lastHandlerResult = new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
      ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, insertionOffset);
      handleEvent = true;
    } else {
      if (logger.isDebugEnabled()) {
        logger.debug("Fragments cannot be inserted in a para element.");
      }
    }
    return handleEvent;
  }
  /**
   * Try to handle invalid insertion events in a html 'table', CALS 'tgroup', 'thead', 'tbody', 'tfoot'. A row element will
   * be inserted (either 'row' or 'tr') with a new cell (either 'entry' or 'td') in which the fragments will be inserted.
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInTable(
      int offset,
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside a tbody, tfoot, thead or html table. We will try to wrap the fragment into a new cell and
    // insert it inside a new row.
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a new row element with a cell.
    AuthorNode tablePart = authorAccess.getDocumentController().getNodeAtOffset(offset);
    boolean isCALS = isElementWithClass(tablePart, TABLE_BODY_CLASS) || isElementWithClass(tablePart, TABLE_HEAD_CLASS);
    boolean isSimpletable = false;
    if (!isCALS) {
      isSimpletable = isElementWithClass(tablePart, SIMPLE_TABLE_CLASS);
    }
    
    String tableRowName = null;
    // Table row name
    if (isCALS) {
      tableRowName = getElementName(authorSchemaManager, context, CALS_TABLE_ROW_CLASS);
    } else if (isSimpletable){
      tableRowName = getElementName(authorSchemaManager, context, SIMPLE_TABLE_ROW_CLASS);
    } else {
      tableRowName = getElementName(authorSchemaManager, context, REL_TABLE_ROW_CLASS);
    }
    boolean canInsertRow = authorSchemaManager.canInsertDocumentFragments(
        new AuthorDocumentFragment[] { 
            authorAccess.getDocumentController().createNewDocumentFragmentInContext(
                "<" + tableRowName + "/>", 
                offset)
            }, 
        context, 
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
    if (canInsertRow) {
      pushContextElement(context, tableRowName);
      String tableEntryName = null;
      // Table entry name
      if (isCALS) {
        tableEntryName = getElementName(authorSchemaManager, context, CALS_TABLE_ENTRY_CLASS);
      } else if (isSimpletable){
        tableEntryName = getElementName(authorSchemaManager, context, SIMPLE_TABLE_ENTRY_CLASS);
      } else {
        tableEntryName = getElementName(authorSchemaManager, context, REL_TABLE_ENTRY_CLASS);
      }
      pushContextElement(context, tableEntryName);

      // Test if fragments can be inserted in the new context.
      if (authorSchemaManager.canInsertDocumentFragments(
          fragmentsToInsert, 
          context, 
          AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {

        // Insert a new row with a cell.
        StringBuilder xmlFragment = new StringBuilder("<");
        xmlFragment.append(tableRowName);

        xmlFragment.append("><");
        xmlFragment.append(tableEntryName);

        xmlFragment.append("/></");
        xmlFragment.append(tableRowName);

        xmlFragment.append(">");
        if (logger.isDebugEnabled()) {
          logger.debug("Insert " + xmlFragment);
        }
        authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

        // Get the newly inserted cell.
        int insertionOffset = -1;
        AuthorNode newCell = authorAccess.getDocumentController().getNodeAtOffset(offset + 2);            
        for (int i = 0; i < fragmentsToInsert.length; i++) { 
          if(insertionOffset == -1) {
            insertionOffset = newCell.getEndOffset();
          }
          authorAccess.getDocumentController().insertFragment(newCell.getEndOffset(), fragmentsToInsert[i]);
        }

        lastHandlerResult = new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
        ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, insertionOffset);
        handleEvent = true;
      } else {
        if (logger.isDebugEnabled()) {
          logger.debug("Fragments cannot be inserted in a para element.");
        }
      }
    }
    return handleEvent;
  }
}
