/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.conref;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;

import javax.xml.transform.sax.SAXSource;

import org.apache.log4j.Logger;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;

import ro.sync.ecss.dita.DITAAccess;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.ValidatingAuthorReferenceResolver;
import ro.sync.ecss.extensions.api.ValidatingReferenceResolverException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocument;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.exml.workspace.api.util.XMLReaderWithGrammar;

/**
 * Resolver for content referred using <code>conref</code> attribute.
 */
public class DITAConRefResolver implements ValidatingAuthorReferenceResolver {

  /**
   * Logger for logging.
   */
  private static Logger logger = Logger.getLogger(DITAConRefResolver.class.getName());
  
  /**
   * The <code>conref</code> attribute.
   */
  private static final String ATTRIBUTE_CONREF = "conref";
  
  /**
   * The <code>conrefend</code> attribute.
   */
  private static final String ATTRIBUTE_CONREF_END = "conrefend";
  
  /**
   * The <code>conkeyref</code> attribute.
   */
  private static final String ATTRIBUTE_CONKEYREF = "conkeyref";
  
  /**
   * Caches access to the grammar pool
   */
  private Object grammarCache = null;

  /**
   * An element that has <code>conref</code> attribute 
   * is an element with references.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#hasReferences(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public boolean hasReferences(AuthorNode node) {
    boolean hasReferences = false;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      hasReferences = attrValue != null;
      if(! hasReferences) {
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        hasReferences = attrValue != null;
      }
    }
    return hasReferences;
  }
  
  /**
   * Returns the value of the <code>conref</code> attribute.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getDisplayName(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public String getDisplayName(AuthorNode node) {
    StringBuilder displayName = new StringBuilder("conref-fragment");
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      if (attrValue != null) {
        displayName.setLength(0);
        displayName.append(attrValue.getValue());
      } else {
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        if (attrValue != null) {
          displayName.setLength(0);
          displayName.append(attrValue.getValue());
        }
      }
      attrValue = element.getAttribute(ATTRIBUTE_CONREF_END);
      if (attrValue != null) {
        displayName.insert(0, '[');
        displayName.append(" .. ").append(attrValue.getValue()).append(']');
        
      }
    }
    return displayName.toString();
  }

  /**
   * Resolve the content referred by <code>conref</code> attribute.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#resolveReference(ro.sync.ecss.extensions.api.node.AuthorNode, java.lang.String, AuthorAccess, EntityResolver)
   */
  public SAXSource resolveReference(
      AuthorNode node, 
      String systemID, 
      AuthorAccess authorAccess,
      EntityResolver  entityResolver) {
    SAXSource saxSource = null;
    
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      try {
        AuthorElement element = (AuthorElement) node;
        DITAConRef conRef = null;
        boolean conKeyRef = false;
        AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
        if(attrValue != null) {
          String attrStringVal = attrValue.getValue();
          conRef = parseDITAConkeyRef(attrStringVal);
          conKeyRef = true;
        } else {
          attrValue = element.getAttribute(ATTRIBUTE_CONREF);
          String attrStringVal = attrValue.getValue();
          conRef = parseDITAHref(systemID, attrStringVal, authorAccess, false);
        }
        if (conRef != null && conRef.getTopicID() != null && conRef.getTopicID().length() > 0) {
          String[] topicPath = getTopicPath(conRef.getTopicID());            
          if (topicPath != null && topicPath.length > 0) {
            InputSource inputSource = entityResolver.resolveEntity(null, conRef.getUri());
            if(inputSource == null) {
              inputSource = new InputSource(conRef.getUri());
            }

            //EXM-16083 XML Reader with grammar cache
            XMLReaderWithGrammar readerWithCache = authorAccess.getXMLUtilAccess().newNonValidatingXMLReader(grammarCache);
            XMLReader xmlReader = readerWithCache != null ? readerWithCache.getXmlReader() : null;
            grammarCache = readerWithCache != null ? readerWithCache.getGrammarCache() : null;
            xmlReader.setEntityResolver(entityResolver);

            //EXM-15209 Expand conref ranges
            String[] endRangePath = null;
            AttrValue conrefEndVal = element.getAttribute(ATTRIBUTE_CONREF_END);
            if (conrefEndVal != null) {
              String conrefEndStr = conrefEndVal.getValue();
              DITAConRef conRefEnd = parseDITAHref(systemID, conrefEndStr, authorAccess, false);
              if(conRefEnd != null && conRefEnd.getTopicID() != null && conRefEnd.getTopicID().length() > 0) {
                endRangePath = getTopicPath(conRefEnd.getTopicID());
                if(conKeyRef) {
                  //If we have "conrefend" with "conkeyref", only the last ID in the path matters.
                  if(endRangePath != null && endRangePath.length > 0) {
                    endRangePath = new String[] {endRangePath[endRangePath.length - 1]};
                  }
                }
              }
            }

            DITAXMLReaderWrapper readerWrapper = 
              new DITAXMLReaderWrapper(
                  xmlReader, 
                  topicPath, 
                  endRangePath);
            saxSource = new SAXSource(readerWrapper, inputSource);
          }
        }
      } catch (MalformedURLException e) {
        logger.error(e, e);
      } catch (SAXNotRecognizedException e) {
        logger.error(e, e);
      } catch (SAXNotSupportedException e) {
        logger.error(e, e);
      } catch (SAXException e) {
        logger.error(e, e);
      } catch (IOException e) {
        logger.error(e, e);
      }
    }

    return saxSource;
  }
  
  /**
   * Gets the topic IDs as an array of strings.
   * 
   * @param value The <code>conref</code> attribute value.
   * @return The path of IDs as an array of strings.
   */
  protected String[] getTopicPath(String value) {
    String[] path = null;
    if(value != null) {
      StringTokenizer tokenizer = new StringTokenizer(value, "/", false);
      path = new String[tokenizer.countTokens()];
      int i = 0;
      while (tokenizer.hasMoreTokens()) {
        path[i++] = tokenizer.nextToken();
      }
    }
    return path;
  }
  
  /**
   * Parse the DITA conkeyref value
   * @param hrefValue The attribute 'conkeyref' value
   * @return The DITA Conref object
   * @throws MalformedURLException
   */
  private DITAConRef parseDITAConkeyRef(String hrefValue) throws MalformedURLException {
    URL absoluteUrl = DITAAccess.resolveKeyRef(hrefValue);
    if(absoluteUrl != null) {
      String id = null;
      String absURL = absoluteUrl.toString();
      String url = absURL; 
      int index = absURL.indexOf("#");
      if(index != -1 && index < absURL.length() - 1) {
        url = absURL.substring(0, index);
        id = absURL.substring(index + 1);
      }
      return new DITAConRef(url, id);
    }
    return null;
  }

  /**
   * Parse the <code>href</code> attribute and returns the absolute URL and the id.
   * 
   * @param baseUrl The base document URL.
   * @param hrefValue The <code>href</code> attribute value.
   * @param authorAccess The author access.
   * Provides access to specific informations and actions for 
   * editor, document, workspace, tables, change tracking, utility 
   * 
   * @return A container object with the content reference URI and topicID or <code>null</code>.
   * 
   * @throws MalformedURLException Can't create the URL.
   */
  protected final DITAConRef parseDITAHref(String baseUrl, String hrefValue, AuthorAccess authorAccess, boolean forTopicRef) throws MalformedURLException {
    DITAConRef toReturn = null;
    // It refers an topic using its ID.
    int index = hrefValue.indexOf('#');
    if (index == hrefValue.length() - 1) {
      // The '#' is the last character (invalid case)
    } else {
      String relativeLocation = baseUrl;
      String id = hrefValue;
      if (index != -1) {
        // Parse the referred file and obtain the title.
        relativeLocation = hrefValue.substring(0, index);
        id = hrefValue.substring(index + 1);
      } else {
        if(forTopicRef) {
          //Points to another file
          relativeLocation = hrefValue;
          id = null;
        } else {
          // It refers the root topic.
        }
      }
      
      //Pass through URI resolver and make absolute relative to base...
      URL absoluteUrl = authorAccess.getXMLUtilAccess().resolvePathThroughCatalogs(new URL(baseUrl), relativeLocation, false, true);
      
      toReturn = new DITAConRef(absoluteUrl.toString(), id);
    }
  
    return toReturn;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Resolves the 'conkeyref/conref' references";
  }

  /**
   * The value of <code>conref</code> attribute is used as the unique identifier.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getReferenceUniqueID(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  public String getReferenceUniqueID(AuthorNode node) {
    String id = null;
    if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
      if (attrValue != null) {
        id = attrValue.getValue();
      } else {
        attrValue = element.getAttribute(ATTRIBUTE_CONREF);
        if (attrValue != null) {
          id = attrValue.getValue();
        }
      }
    }
    return id;
  }

  /**
   * Returns <code>true</code> when the attribute name is equal to <code>'conref'</code>.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#isReferenceChanged(ro.sync.ecss.extensions.api.node.AuthorNode, java.lang.String)
   */
  public boolean isReferenceChanged(AuthorNode node, String attributeName) {
    return ATTRIBUTE_CONKEYREF.equals(attributeName) || ATTRIBUTE_CONREF.equals(attributeName) || ATTRIBUTE_CONREF_END.equals(attributeName);
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorReferenceResolver#getReferenceSystemID(ro.sync.ecss.extensions.api.node.AuthorNode, AuthorAccess)
   */
  public String getReferenceSystemID(AuthorNode node, AuthorAccess authorAccess) {
    String systemID = null;
    try {
      if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
        AuthorElement element = (AuthorElement) node;
        AttrValue attrValue = element.getAttribute(ATTRIBUTE_CONKEYREF);
        if (attrValue != null) {
          String attrStringVal = attrValue.getValue();
          URL absURL = DITAAccess.resolveKeyRef(attrStringVal);
          if(absURL != null) {
            systemID = absURL.toString();
          }
        } 
        if(systemID == null) {
          attrValue = element.getAttribute(ATTRIBUTE_CONREF);
          if (attrValue != null) {
            String attrStringVal = attrValue.getValue();
            systemID = new URL(node.getXMLBaseURL(), attrStringVal).toString();

            //Try to pass through catalog.
            DITAConRef parseDITAHref = parseDITAHref(
                node.getXMLBaseURL()  == null ? null : node.getXMLBaseURL().toString(), 
                    attrStringVal, authorAccess, false);
            if(parseDITAHref != null) {
              systemID = parseDITAHref.getUri();
              if(parseDITAHref.getTopicID() != null && parseDITAHref.getTopicID().length() > 0) {
                systemID +="#" + parseDITAHref.getTopicID();
              }
            }
          }
        }
      }
    } catch (MalformedURLException e) {
      if (logger.isDebugEnabled()) {
        logger.debug(e, e);
      }
    }
    return systemID;
  }

  /**
   * 
   * @see ro.sync.ecss.extensions.api.ValidatingAuthorReferenceResolver#checkTarget(ro.sync.ecss.extensions.api.node.AuthorNode, ro.sync.ecss.extensions.api.node.AuthorDocument)
   */
  public void checkTarget(AuthorNode node, AuthorDocument targetDocument) throws ValidatingReferenceResolverException{
    String errorMessage = null;
    AuthorElement targetElement = targetDocument.getRootElement();
    if(targetElement != null) {
      if(! targetElement.getName().equals(node.getName())) {
        AttrValue targetClass = targetElement.getAttribute("class");
        if (targetClass == null) {
          errorMessage = "The target does not have a class attribute";
        } else {
          if (node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
            AuthorElement sourceElement = (AuthorElement) node;
            AttrValue sourceClass = sourceElement.getAttribute("class");
            if(sourceClass != null) {
              boolean generalization = DITAAccess.isGeneralizationOf(sourceClass.getValue(), targetClass.getValue());
              if (!generalization) {
                errorMessage = "The source element \"" + sourceElement.getName() + 
                "\" with class value \"" + sourceClass.getValue() + 
                "\" is not a generalization of target element \"" + targetElement.getName() + 
                "\" with class value \"" + targetClass.getValue() + "\"";
              }
            } else {
              errorMessage = "The source does not have a class attribute";
            }
          } else {
            errorMessage = "The conref source is not an element";
          }
        }
      } else {
        //Same name in target and source, probably OK
      }
    }
    if(errorMessage != null) {
      throw new ValidatingReferenceResolverException("Conref was not expanded:\n" + errorMessage);
    }
  }
}