/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import java.util.List;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.contentcompletion.xml.ContextElement;
import ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter;
import ro.sync.ecss.extensions.api.AuthorSchemaManager;
import ro.sync.ecss.extensions.api.InvalidEditException;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResult;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResultsImpl;

/**
 * Specific editing support for Docbook documents. Handles typing and paste events inside itemizedlist, orderlist (a 
 * listitem with a para will be created and the typing/paste content be put inside it) and tables.
 */
public class DocbookSchemaAwareEditingHandler extends AuthorSchemaAwareEditingHandlerAdapter {
  /** 
   * Logger for logging. 
   */
  private static Logger logger = Logger.getLogger(DocbookSchemaAwareEditingHandler.class.getName());
  /**
   * For Docbook 5 http://docbook.org/ns/docbook, for Docbook 4 an empty string.
   */
  protected final String documentNamespace;
  /**
   * Docbook itemized list element name.
   */
  private static final String ITEMIZED_LIST = "itemizedlist";
  /**
   * Docbook list item element name.
   */
  private static final String LIST_ITEM = "listitem";
  /**
   * Docbook sect1 element name.
   */
  private static final String SECT1 = "sect1";
  /**
   * Docbook sect2 element name.
   */
  private static final String SECT2 = "sect2";
  /**
   * Docbook sect3 element name.
   */
  private static final String SECT3 = "sect3";
  /**
   * Docbook sect4 element name.
   */
  private static final String SECT4 = "sect4";
  /**
   * Docbook sect5 element name.
   */
  private static final String SECT5 = "sect5";
  /**
   * Docbook section element name.
   */
  private static final String SECTION = "section";
  /**
   * Docbook ordered list element name.
   */
  private static final String ORDERED_LIST = "orderedlist";
  /**
   * Docbook CALS table element name.
   */
  private static final String HTML_TABLE = "table";
  private static final String CALS_TABLE = "tgroup";
  private static final String TABLE_HEAD = "thead";
  private static final String TABLE_FOOT = "tfoot";
  private static final String TABLE_BODY = "tbody";
  private static final String CALS_TABLE_ROW = "row";
  private static final String CALS_TABLE_ENTRY = "entry";
  private static final String HTML_TABLE_ROW = "tr";
  private static final String HTML_TABLE_ENTRY = "td";
  /**
   * Docbook HTML para element name.
   */
  protected static final String PARA = "para";
  /**
   * Docbook HTML title element name.
   */
  protected static final String TITLE = "title";
  /**
   * Docbook info element name suffix.
   */
  protected static final String INFO_SUFIX = "info";
  
  /**
   * Constructor.
   * 
   * @param documentNamespace The document namespace, for different versions of Docbook. 
   */
  public DocbookSchemaAwareEditingHandler(String documentNamespace) {
    this.documentNamespace = documentNamespace;
  }
  /**
   * @see ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter#handleTyping(int, char, ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public boolean handleTyping(int offset, char ch, AuthorAccess authorAccess) throws InvalidEditException {
    boolean handleTyping = false;
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    if (!authorSchemaManager.isLearnSchema() && 
        !authorSchemaManager.hasLoadingErrors() &&
        authorSchemaManager.getAuthorSchemaAwareOptions().isEnableSmartTyping()) {
      try {
        AuthorDocumentFragment characterFragment = 
          authorAccess.getDocumentController().createNewDocumentTextFragment(String.valueOf(ch));    
        handleTyping = handleInsertionEvent(offset, new AuthorDocumentFragment[] {characterFragment}, authorAccess);
      } catch (AuthorOperationException e) {
        throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
      }
    }
    return handleTyping;    
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandlerAdapter#handlePasteFragment(int, ro.sync.ecss.extensions.api.node.AuthorDocumentFragment[], int, ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public boolean handlePasteFragment(int offset, AuthorDocumentFragment[] fragmentsToInsert,
      int actionId, AuthorAccess authorAccess) throws InvalidEditException {
    boolean handleInsertionEvent = false;
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    if (!authorSchemaManager.isLearnSchema() && 
        !authorSchemaManager.hasLoadingErrors() &&
        authorSchemaManager.getAuthorSchemaAwareOptions().isEnableSmartPaste()) {
      handleInsertionEvent = handleInsertionEvent(offset, fragmentsToInsert, authorAccess);
    }
    return handleInsertionEvent;
  }
  
  /**
   * Handle an insertion event (either typing or paste).
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   * 
   * @throws InvalidEditException The event was rejected because it is invalid.
   */
  private boolean handleInsertionEvent(
      int offset, 
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess) throws InvalidEditException {
    AuthorSchemaManager authorSchemaManager = authorAccess.getDocumentController().getAuthorSchemaManager();
    boolean handleEvent = false;
    try {
      // The fragment can't just be inserted at the given offset.
      AuthorNode nodeAtInsertionOffset = authorAccess.getDocumentController().getNodeAtOffset(offset);
      if (logger.isDebugEnabled()) {
        logger.debug("nodeAtInsertionOffset " + nodeAtInsertionOffset);
      }
      if(isElementWithNameAndNamespace(nodeAtInsertionOffset, DocbookSchemaAwareEditingHandler.ITEMIZED_LIST) 
          || isElementWithNameAndNamespace(nodeAtInsertionOffset, DocbookSchemaAwareEditingHandler.ORDERED_LIST)) {
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInLists(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if(isElementWithNameAndNamespace(nodeAtInsertionOffset, LIST_ITEM) && 
          nodeAtInsertionOffset.getStartOffset() + 1 == nodeAtInsertionOffset.getEndOffset()) {
        // Insertion in empty 'listitem'
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInListItem(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if (isElementWithNameAndNamespace(nodeAtInsertionOffset, TABLE_BODY) 
          || isElementWithNameAndNamespace(nodeAtInsertionOffset, TABLE_FOOT)
          || isElementWithNameAndNamespace(nodeAtInsertionOffset, TABLE_HEAD)
          || isHTMLTable(nodeAtInsertionOffset)) {
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInTable(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } else if((isElementWithNameAndNamespace(nodeAtInsertionOffset, SECT1) || 
          isElementWithNameAndNamespace(nodeAtInsertionOffset, SECT2) ||
          isElementWithNameAndNamespace(nodeAtInsertionOffset, SECT3) ||
          isElementWithNameAndNamespace(nodeAtInsertionOffset, SECT4) ||
          isElementWithNameAndNamespace(nodeAtInsertionOffset, SECT5) ||
          isElementWithNameAndNamespace(nodeAtInsertionOffset, SECTION))) {
        
        // Insertion in empty 'sect1'
        // Check if the fragment is allowed as it is.
        boolean canInsertFragments = authorSchemaManager.canInsertDocumentFragments(
            fragmentsToInsert, 
            offset, 
            AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
        if (!canInsertFragments) {
          handleEvent = handleInvalidInsertionEventInSect(
              offset, 
              fragmentsToInsert, 
              authorAccess,
              authorSchemaManager);
        }
      } 
      
    } catch (BadLocationException e) {
      throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
    } catch (AuthorOperationException e) {
      throw new InvalidEditException(e.getMessage(), "Invalid typing event: " + e.getMessage(), e, false);
    }
    return handleEvent;    
  }
  
  /**
   * Verify if a given node represents a html table.
   * 
   * @param node The author node.
   * @return  <code>true</code> if the given node represents a html table.
   */
  private boolean isHTMLTable(AuthorNode node) {
    boolean htmlTable = false;
    if(isElementWithNameAndNamespace(node, HTML_TABLE)) {
      htmlTable = true;
      AuthorElement element = (AuthorElement) node;
      List<AuthorNode> contentNodes = element.getContentNodes();
      // If there is a 'tbody' child node the given node represents a cals table. 
      for (AuthorNode authorNode : contentNodes) {
        if (isElementWithNameAndNamespace(authorNode, CALS_TABLE)) {
          htmlTable = false;
          break;
        }
      }
    }
    return htmlTable;
  }
  
  /**
   * Try to handle invalid insertion events in 'section', 'sect1', 'sect2', 'sect3', 'sect4' or 'sect5'. 
   * The solution is to insert the <code>fragmentsToInsert</code> into a 'title' element if the sect element is empty or
   * into a 'para' element if the sect already contains a 'title' or 'info' element.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInSect(int offset, AuthorDocumentFragment[] fragmentsToInsert, AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside an itemized section, sect1, sect2, sect3, sect4 or sect5. 
    // We will try to insert a sect element with title and para inside 
    // and perform the insertion inside the para element.
    AuthorElement sectElement = (AuthorElement) authorAccess.getDocumentController().getNodeAtOffset(offset);
    
    if (sectElement.getStartOffset() + 1 == sectElement.getEndOffset()) {
      // Empty sect element
      WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
      // Derive the context by adding a list item with a title inside.
      pushContextElement(context, TITLE);

      // Test if fragments can be inserted in 'title' element
      if (authorSchemaManager.canInsertDocumentFragments(
          fragmentsToInsert, 
          context, 
          AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
        // Create a title structure and insert fragments inside
        StringBuilder xmlFragment = new StringBuilder("<").append(DocbookSchemaAwareEditingHandler.TITLE);
        if (documentNamespace != null && documentNamespace.length() != 0) {
          xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
        }
        xmlFragment.append(">").append("</").append(DocbookSchemaAwareEditingHandler.TITLE).append(">");
        if (logger.isDebugEnabled()) {
          logger.debug("Insert " + xmlFragment);
        }
        // Insert title
        authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

        // Insert fragments
        AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 1);
        int insertionOffset = -1;
        for (int i = 0; i < fragmentsToInsert.length; i++) {
          if(insertionOffset == -1) {
            insertionOffset = newParaNode.getEndOffset();
          }
          authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
        }
        lastHandlerResult = 
          new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
        ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(
            SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, 
            insertionOffset);
        handleEvent = true;
      } else {
        if (logger.isDebugEnabled()) {
          logger.debug("Fragments cannot be inserted in a para element.");
        }
      }
    } else {
      // Check if there is just a title.
      List<AuthorNode> contentNodes = sectElement.getContentNodes();
      if (contentNodes.size() == 1) {
        AuthorNode child = contentNodes.get(0);
        boolean isTitleChild = isElementWithNameAndNamespace(child, TITLE) || isElementWithNameAndNamespace(child, 
            getInfoElementChildOfSect(sectElement.getLocalName()));
        if (isTitleChild && child.getEndOffset() < offset) {
          // We are after the title or info.

          // Empty sect element
          WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
          // Derive the context by adding a list item with a title inside.
          pushContextElement(context, PARA);

          // Test if fragments can be inserted in 'title' element
          if (authorSchemaManager.canInsertDocumentFragments(
              fragmentsToInsert, 
              context, 
              AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
            // Create a title structure and insert fragments inside
            StringBuilder xmlFragment = new StringBuilder("<").append(PARA);
            if (documentNamespace != null && documentNamespace.length() != 0) {
              xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
            }
            xmlFragment.append(">").append("</").append(PARA).append(">");
            // Insert title
            authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

            // Insert fragments
            int insertionOffset = -1;
            AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 1);            
            for (int i = 0; i < fragmentsToInsert.length; i++) {
              if(insertionOffset == -1) {
                insertionOffset = newParaNode.getEndOffset();
              }
              authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
            }            
            lastHandlerResult = 
              new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
            ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(
                SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, 
                insertionOffset);
            
            handleEvent = true;
          } else {
            if (logger.isDebugEnabled()) {
              logger.debug("Fragments cannot be inserted in a para element.");
            }
          }
        }
      }
    }
    return handleEvent;
  }
  
  /**
   * Get the info child element name of to the given sect element name.
   *  
   * @param sectElementName The sect element name.
   * @return  The info child element name.
   */
  protected String getInfoElementChildOfSect(String sectElementName) {
    return sectElementName.concat(INFO_SUFIX);
  }
  
  /**
   * Try to handle invalid insertion events in 'itemizedlist' or 'orderedlist'. The fallback is to insert 
   * the <code>fragmentsToInsert</code> into a 'listitem/para' structure if is possible.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInLists(int offset,
      AuthorDocumentFragment[] fragmentsToInsert, AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside an itemized list or ordered list. We will try to insert a list item with a para inside 
    // and perform the insertion inside the para element.
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a list item with a para inside.
    pushContextElement(context, DocbookSchemaAwareEditingHandler.LIST_ITEM);
    pushContextElement(context, PARA);
    // Test if fragments can be inserted in 'para' element
    if (authorSchemaManager.canInsertDocumentFragments(
        fragmentsToInsert, 
        context, 
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
      // Create a listitem/para structure and insert fragments inside the new para
      StringBuilder xmlFragment = new StringBuilder("<").append(DocbookSchemaAwareEditingHandler.LIST_ITEM);
      if (documentNamespace != null && documentNamespace.length() != 0) {
        xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
      }
      xmlFragment.append("><").append(PARA).append("/></").append(DocbookSchemaAwareEditingHandler.LIST_ITEM).append(">");
      if (logger.isDebugEnabled()) {
        logger.debug("Insert " + xmlFragment);
      }
      // Insert listitem/para
      authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

      // Insert fragments
      int insertionOffset = -1;
      AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 2);            
      for (int i = 0; i < fragmentsToInsert.length; i++) { 
        if(insertionOffset == -1) {
          insertionOffset = newParaNode.getEndOffset();
        }
        authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
      }
      
      lastHandlerResult = 
        new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
      ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(
          SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, 
          insertionOffset);

      handleEvent = true;
    } else {
      if (logger.isDebugEnabled()) {
        logger.debug("Fragments cannot be inserted in a para element.");
      }
    }
    return handleEvent;
  }
  
  /**
   * Derive the given context by adding the given element.
   */
  protected void pushContextElement(WhatElementsCanGoHereContext context, String elementName) {
    ContextElement contextElement = new ContextElement();
    contextElement.setQName(elementName);
    contextElement.setNamespace(documentNamespace);
    context.pushContextElement(contextElement, null);
  }
  
  /**
   * @return <code>true</code> if the given node is an element with the given local name and from the Docbook namespace.
   */
  protected boolean isElementWithNameAndNamespace(AuthorNode node, String elementLocalName) {
    boolean result = false;
    if(node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement element = (AuthorElement) node;
      result = elementLocalName.equals(element.getLocalName()) && element.getNamespace().equals(documentNamespace);
    }
    return result;
  }
  /**
   * Try to handle invalid insertion events in 'listitem'. The fallback is to insert 
   * the <code>fragmentsToInsert</code> into a 'para' element if is possible.  
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInListItem(
      int offset,
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside a listitem. We will try to wrap the fragment into a new para element
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a list item with a para inside.
    pushContextElement(context, PARA);
    // Test if fragments can be inserted in 'para' element
    if (authorSchemaManager.canInsertDocumentFragments(
        fragmentsToInsert, 
        context, 
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {
      
      // Insert 'para' in listitem
      StringBuilder xmlFragment = new StringBuilder("<").append(DocbookSchemaAwareEditingHandler.PARA);
      if (documentNamespace != null && documentNamespace.length() != 0) {
        xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
      }
      xmlFragment.append("/>");
      if (logger.isDebugEnabled()) {
        logger.debug("Insert " + xmlFragment);
      }
      authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);
  
      // Get the newly inserted para element.
      AuthorNode newParaNode = authorAccess.getDocumentController().getNodeAtOffset(offset + 1);            
      int insertionOffset = -1;
      for (int i = 0; i < fragmentsToInsert.length; i++) { 
        if(insertionOffset == -1) {
          insertionOffset = newParaNode.getEndOffset();
        }
        authorAccess.getDocumentController().insertFragment(newParaNode.getEndOffset(), fragmentsToInsert[i]);
      }
 
      lastHandlerResult = 
        new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
      ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(
          SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, 
          insertionOffset);

      handleEvent = true;
    } else {
      if (logger.isDebugEnabled()) {
        logger.debug("Fragments cannot be inserted in a para element.");
      }
    }
    return handleEvent;
  }
  /**
   * Try to handle invalid insertion events in a html 'table', CALS 'tgroup', 'thead', 'tbody', 'tfoot'. A row element will
   * be inserted (either 'row' or 'tr') with a new cell (either 'entry' or 'td') in which the fragments will be inserted.
   * 
   * @param offset Offset where the insertion event occurred.
   * @param fragmentsToInsert Fragments that must be inserted at the given offset. 
   * @param authorAccess Author access.
   * @return <code>true</code> if the event was handled, <code>false</code> otherwise.
   */
  private boolean handleInvalidInsertionEventInTable(
      int offset,
      AuthorDocumentFragment[] fragmentsToInsert, 
      AuthorAccess authorAccess,
      AuthorSchemaManager authorSchemaManager) throws BadLocationException, AuthorOperationException {
    boolean handleEvent = false;
    // Typing/paste inside a tbody, tfoot, thead or html table. We will try to wrap the fragment into a new cell and
    // insert it inside a new row.
    WhatElementsCanGoHereContext context = authorSchemaManager.createWhatElementsCanGoHereContext(offset);
    // Derive the context by adding a new row element with a cell.
    AuthorNode tablePart = authorAccess.getDocumentController().getNodeAtOffset(offset);
    AuthorNode table = ((AuthorElement) tablePart).getParent();
    boolean isCALS = isElementWithNameAndNamespace(table, CALS_TABLE);
    
    StringBuilder xmlFragment = new StringBuilder("<");
    if (isCALS) {
      xmlFragment.append(DocbookSchemaAwareEditingHandler.CALS_TABLE_ROW);
    } else {
      xmlFragment.append(DocbookSchemaAwareEditingHandler.HTML_TABLE_ROW);
    }
    if (documentNamespace != null && documentNamespace.length() != 0) {
      xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
    }
    xmlFragment.append("/>");
    
    // Check if a row can be inserted at the current offset.
    boolean canInsertRow = authorSchemaManager.canInsertDocumentFragments(
        new AuthorDocumentFragment[] {authorAccess.getDocumentController().createNewDocumentFragmentInContext(xmlFragment.toString(), offset)},
        context,
        AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS);
    
    if (canInsertRow) {
      if (isCALS) {
        pushContextElement(context, CALS_TABLE_ROW);
        pushContextElement(context, CALS_TABLE_ENTRY);
      } else {
        pushContextElement(context, HTML_TABLE_ROW);
        pushContextElement(context, HTML_TABLE_ENTRY);
      }
      // Test if fragments can be inserted in the new context.
      if (authorSchemaManager.canInsertDocumentFragments(
          fragmentsToInsert, 
          context, 
          AuthorSchemaManager.VALIDATION_MODE_STRICT_FIRST_CHILD_LAX_OTHERS)) {

        // Insert a new row with a cell.
        xmlFragment = new StringBuilder("<");
        if (isCALS) {
          xmlFragment.append(DocbookSchemaAwareEditingHandler.CALS_TABLE_ROW);
        } else {
          xmlFragment.append(DocbookSchemaAwareEditingHandler.HTML_TABLE_ROW);
        }
        if (documentNamespace != null && documentNamespace.length() != 0) {
          xmlFragment.append(" xmlns=\"").append(documentNamespace).append("\"");
        }
        xmlFragment.append("><");
        if (isCALS) {
          xmlFragment.append(CALS_TABLE_ENTRY);
        } else {
          xmlFragment.append(HTML_TABLE_ENTRY);
        }
        xmlFragment.append("/></");
        if (isCALS) {
          xmlFragment.append(DocbookSchemaAwareEditingHandler.CALS_TABLE_ROW);
        } else {
          xmlFragment.append(DocbookSchemaAwareEditingHandler.HTML_TABLE_ROW);
        }
        xmlFragment.append(">");
        if (logger.isDebugEnabled()) {
          logger.debug("Insert " + xmlFragment);
        }
        authorAccess.getDocumentController().insertXMLFragment(xmlFragment.toString(), offset);

        // Get the newly inserted cell.
        AuthorNode newCell = authorAccess.getDocumentController().getNodeAtOffset(offset + 2);            
        int insertionOffset = -1;
        for (int i = 0; i < fragmentsToInsert.length; i++) { 
          if(insertionOffset == -1) {
            insertionOffset = newCell.getEndOffset();
          }
          authorAccess.getDocumentController().insertFragment(newCell.getEndOffset(), fragmentsToInsert[i]);
        }

        lastHandlerResult = 
          new SchemaAwareHandlerResultsImpl(SchemaAwareHandlerResult.TYPE_HANDLE_INSERT_FRAGMENT_OPERATION);
        ((SchemaAwareHandlerResultsImpl)lastHandlerResult).addResult(
            SchemaAwareHandlerResult.RESULT_ID_HANDLE_INSERT_FRAGMENT_OFFSET, 
            insertionOffset);

        handleEvent = true;
      } else {
        if (logger.isDebugEnabled()) {
          logger.debug("Fragments cannot be inserted in a para element.");
        }
      }
    }
    return handleEvent;
  }
}
