/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentType;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.access.AuthorWorkspaceAccess;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.images.ImageHandlerDispatcher;
import ro.sync.ecss.images.xmlimages.XMLImageHandler;
import ro.sync.ecss.images.xmlimages.AbstractXMLImageHandler.CannotEditException;

/**
 * Operation used to insert an equation in Docbook documents.
 */
public class InsertEquationOperation implements AuthorOperation {

  /**
   * Docbook namespace.
   */
  private static final String DOCBOOK_NS = "http://docbook.org/ns/docbook";
  
  /**
   * The MathML fragment representing the default equation.
   */
  private static String MATH_ML = 
      "<mml:math xmlns:mml='http://www.w3.org/1998/Math/MathML'><mml:mrow>" + 
      "<mml:msup><mml:mi>a</mml:mi><mml:mn>2</mml:mn></mml:msup>" +
      "<mml:mo>=</mml:mo>" +
      "<mml:msup><mml:mi>b</mml:mi><mml:mn>2</mml:mn></mml:msup>" +
      "<mml:mo>+</mml:mo>" +
      "<mml:msup><mml:mi>c</mml:mi><mml:mn>2</mml:mn>" + 
      "</mml:msup></mml:mrow></mml:math>";

  /**
  * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
  */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    try {
      AuthorWorkspaceAccess workspace = authorAccess.getWorkspaceAccess(); 
      if (!workspace.isStandalone()) {
        //We do not support editing in EC
        workspace.showInformationMessage("MathML editing is not supported on Eclipse.");
      } else {
        //Start editing the equation
        XMLImageHandler handler = ImageHandlerDispatcher.getInstance()
            .getXMLImageHandlerFor("mathml");
        if(handler != null) {
          String serializedDoctype = null;
          AuthorDocumentType dt = authorAccess.getDocumentController().getDoctype();
          if(dt != null){
            serializedDoctype = dt.getContent();
          }
          
          String mml = handler.editImage(MATH_ML,
              serializedDoctype,
              authorAccess.getEditorAccess().getEditorLocation().toString());
          
          if(mml != null){          
            int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
            
            // Determine if it is Docbook 5.
            // Go up to find an element.        
            boolean addNSDecl = false;
            AuthorNode node = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);          
            while (node != null && node.getType() != AuthorNode.NODE_TYPE_ELEMENT ){
              node = node.getParent();
            }
            if(node != null){
              if(DOCBOOK_NS.equals(((AuthorElement)node).getNamespace())){
                // It is Docbook 5.
                addNSDecl = true;
              }
            }
            
            boolean inInlineContext = authorAccess.getDocumentController().inInlineContext(caretOffset);
            StringBuffer fragment;
            if (inInlineContext) {
              // InlineEquation
              fragment = new StringBuffer("<inlineequation");
              if(addNSDecl){
                fragment.append(" xmlns='").append(DOCBOOK_NS).append("'>");
              } else {
                fragment.append('>');
              }
              fragment.append(mml);
              fragment.append("</inlineequation>");
            } else {
              // Equation
              fragment = new StringBuffer("<equation");
              if(addNSDecl){
                fragment.append(" xmlns='").append(DOCBOOK_NS).append("'>");
              } else {
                fragment.append('>');
              }
              fragment.append(mml);
              fragment.append("</equation>");
            }
            
            // Inserts the equation
            authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
                fragment.toString(),
                caretOffset);
          }
        } else {
          //Maybe running from the Author component applet
          workspace.showInformationMessage("MathML editing is not supported.");
        }
      }
    } catch (BadLocationException e) {
      throw new AuthorOperationException("The operation cannot be performed due to: "
          + e.getMessage(), e);
    } catch (CannotEditException e) {
      throw new AuthorOperationException(e.getMessage(), e);
    }
  }

  /**
  * No arguments. The operation will display a dialog for choosing the image fileref.
  * 
  * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
  */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
  * @see ro.sync.ecss.extensions.api.Extension#getDescription()
  */
  public String getDescription() {
    return "Insert a MathML equation.";
  }
}