/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.ImageFileChooser;
import ro.sync.util.URLUtil;

/**
 * Operation used to insert a Docbook graphic.
 */
public class InsertGraphicOperation implements AuthorOperation {
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
  throws IllegalArgumentException, AuthorOperationException {
    // Cheoose an image file.
    String ref = ImageFileChooser.chooseImageFile(authorAccess);
    if (ref != null) {
      try {        
        if (authorAccess.getDocumentController().inInlineContext(authorAccess.getEditorAccess().getCaretOffset())) {
          // Insert the inline graphic
          authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
              "<inlinegraphic fileref=\"" + ref + "\"/>" ,
              authorAccess.getEditorAccess().getCaretOffset());
        } else {
          // Insert the graphic
          AuthorNode nodeAtOffset = authorAccess.getDocumentController().getNodeAtOffset(
              authorAccess.getEditorAccess().getCaretOffset());
          boolean figureContext = false;
          // First verify that we are in a figure context or not...
          while (nodeAtOffset != null) {
            if (nodeAtOffset.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
              AuthorElement element = (AuthorElement) nodeAtOffset;
              if ("figure".equals(element.getLocalName())) {
                // Already in a figure element.
                figureContext = true;
                break;
              }
            }
            nodeAtOffset = nodeAtOffset.getParent();
          }
          
          if (figureContext) {
            // In a figure, insert a media object.
            authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
                "            <mediaobject>\n" + 
                "                <imageobject><imagedata fileref=\"" + ref + "\"/></imageobject>\n" + 
                "            </mediaobject>\n", 
                authorAccess.getEditorAccess().getCaretOffset()); 
          } else {
            authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
                // Insert a figure.
                "<figure>\n" + 
                "            <title>" + URLUtil.extractFileName(ref) + "</title>\n" + 
                "            <mediaobject>\n" + 
                "                <imageobject><imagedata fileref=\"" + ref + "\"/></imageobject>\n" + 
                "            </mediaobject>\n" + 
                "        </figure>" ,
                authorAccess.getEditorAccess().getCaretOffset());
          }
        }
      } catch (BadLocationException e) {
        throw new AuthorOperationException("The operation cannot be performed due to: " + e.getMessage(), e);    
      }
    }
  }

  /**
   * No arguments. The operation will display a dialog for choosing the image file.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Insert a docbook graphic";
  }
}