/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.ImageFileChooser;
import ro.sync.util.URLUtil;

/**
 * Operation used to insert an image in Docbook 5 documents.
 */
public class InsertImageDataOperation implements AuthorOperation {
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
  throws IllegalArgumentException, AuthorOperationException {
    try {
      String ref = ImageFileChooser.chooseImageFile(authorAccess);
      if(ref != null) {
        StringBuffer fragment = null;
        boolean inInlineContext = authorAccess.getDocumentController().inInlineContext(
            authorAccess.getEditorAccess().getCaretOffset());
        boolean figureContext = false;
        if (inInlineContext) {
          // The caret is inside an inline context, a paragraph by example. Insert an 'inlinemediaobject'          
          fragment = new StringBuffer("<inlinemediaobject");
          fragment.append(" xmlns=\"http://docbook.org/ns/docbook\"><imageobject><imagedata ");
        } else {
          AuthorNode nodeAtOffset = authorAccess.getDocumentController().getNodeAtOffset(
              authorAccess.getEditorAccess().getCaretOffset());
          while (nodeAtOffset != null) {
            if (nodeAtOffset.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
              AuthorElement element = (AuthorElement) nodeAtOffset;
              if ("figure".equals(element.getLocalName()) && "http://docbook.org/ns/docbook".equals(element.getNamespace())) {
                // Already in a figure element.
                figureContext = true;
                break;
              }
            }
            nodeAtOffset = nodeAtOffset.getParent();
          }
          // The caret is inside an block context, between two paragraphs by example. 
          // Insert a 'figure'/'mediaobject'
          fragment = new StringBuffer();
          if (!figureContext) {
            fragment.append("<figure xmlns=\"http://docbook.org/ns/docbook\">");
            fragment.append("<title>").append(URLUtil.extractFileName(ref)).append("</title>");
            fragment.append("<mediaobject><imageobject><imagedata ");
          } else {
            fragment.append("<mediaobject xmlns=\"http://docbook.org/ns/docbook\"><imageobject><imagedata ");
          }
        }
        fragment.append("fileref=\"");
        fragment.append(ref);
        fragment.append("\"/></imageobject>");
        
        if (inInlineContext) {
          fragment.append("</inlinemediaobject>");
        } else {
          fragment.append("</mediaobject>");
          if (!figureContext) {
            fragment.append("</figure>");
          }
        }
        
        // Insert the graphic
        authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
            fragment.toString(),
            authorAccess.getEditorAccess().getCaretOffset());
      }
    } catch (BadLocationException e) {
      throw new AuthorOperationException(
          "The operation cannot be performed due to: " + e.getMessage(), e);
    }
  }

  /**
   * No arguments. The operation will display a dialog for choosing the image fileref.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Insert a docbook imagedata";
  }
}