/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook.table;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.commons.table.operations.AbstractTableOperation;
import ro.sync.ecss.extensions.commons.table.operations.TableInfo;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizerConstants.ColumnWidthsType;
import ro.sync.ecss.extensions.commons.table.operations.cals.CALSDocumentTypeHelper;

/**
 * Operation used to insert a CALS Docbook table or 
 * an HTML table in a DocBook document. 
 */
public class InsertTableOperation extends AbstractTableOperation {
  
  /**
   * Constructor.
   */
  public InsertTableOperation() {
    super(new CALSDocumentTypeHelper());
  }

  /**
   * Arguments of the operation.
   */
  private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] { NAMESPACE_ARGUMENT_DESCRIPTOR };
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    Object namespaceObj =  args.getArgumentValue(NAMESPACE_ARGUMENT);
    
    // Show the 'Insert table' dialog
    TableInfo tableInfo = null;
    if (authorAccess.getWorkspaceAccess().isStandalone()) {
      tableInfo = new SADocbookTableCustomizer().customizeTable(
          authorAccess, 
          true);
    } else {
      tableInfo = new ECDocbookTableCustomizer().customizeTable(
          authorAccess, 
          true);
    }
    if (tableInfo != null) {
      StringBuffer tableXMLFragment = getXMLFragment(tableInfo, (String) namespaceObj);
      
      // Insert the table schema aware(EXM-16077)
      authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
          tableXMLFragment.toString(), 
          authorAccess.getEditorAccess().getCaretOffset());
    } else {
      // User canceled the operation 
    }
  }

  /**
   * Create the XML fragment for the table to be inserted.
   * 
   * @param tableInfo The object containing information about the table.
   * @param namespace The namespace of the table element. 
   * @return The string buffer containing the XML fragment representing the 
   * table element.
   */
  private StringBuffer getXMLFragment(TableInfo tableInfo, String namespace) {
    // Create the table XML fragment
    StringBuffer tableXMLFragment = new StringBuffer();
    // Table element
    if (tableInfo.getTitle() != null) {
      // Title was specified, insert a table with title
      tableXMLFragment.append("<table ");
    } else {
      // Title was not specified, insert an informaltable        
      tableXMLFragment.append("<informaltable ");
    }
    
    // Add the namespace.
    if (namespace != null) {
      tableXMLFragment.append("xmlns=\"").append(namespace).append("\" ");
    }
    
    // Add the frame.
    tableXMLFragment.append("frame=\"" + tableInfo.getFrame() + "\">");    
    
    if (tableInfo.getTableModel() == TableInfo.TABLE_MODEL_CUSTOM || 
        tableInfo.getTableModel() == TableInfo.TABLE_MODEL_CALS) {
      // CALS table model for DocBook v.4 or DocBook v.5
      addCALSXMLFragment(tableXMLFragment, tableInfo, namespace);
    } else {
      // HTML table model for DocBook v.4 or DocBook v.5
      addHTMLXMLFragment(tableXMLFragment, tableInfo, namespace);
    }
    
    if (tableInfo.getTitle() != null) {
      tableXMLFragment.append("</table>");
    } else {
      tableXMLFragment.append("</informaltable>");
    }
    return tableXMLFragment;
  }
  
  /**
   * Adds to the XML fragment the elements (tgroup, colspec, tbody, thead, etc.) 
   * specific to the CALS table model.
   *  
   * @param tableXMLFragment The string buffer where to append CALS elements.
   * @param tableInfo The table information.
   * @param namespace The table element namespace.
   */
  private void addCALSXMLFragment(StringBuffer tableXMLFragment, TableInfo tableInfo, String namespace) {
    // Title element
    if (tableInfo.getTitle() != null) {
      tableXMLFragment.append("<title>" + tableInfo.getTitle() + "</title>");
    }
    
    // Add 'tgroup', A wrapper for the main content of a table
    tableXMLFragment.append("<tgroup cols=\"" + tableInfo.getColumnsNumber() + "\">");
    
    // Add column specifications
    addCALSColSpecs(tableXMLFragment, tableInfo);
    
    // Add table header
    addCALSTableHeader(tableXMLFragment, tableInfo);
    
    // Add table footer
    addCALSTableFooter(tableXMLFragment, tableInfo);
    
    // Add table body
    addCALSTableBody(tableXMLFragment, tableInfo);
    
    tableXMLFragment.append("</tgroup>");    
  }

  /**
   * Add table body for CALS model.
   */
  private void addCALSTableBody(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    tableXMLFragment.append("<tbody>");
    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<row>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        tableXMLFragment.append("<entry></entry>"); 
      }
      tableXMLFragment.append("</row>"); 
    }
    tableXMLFragment.append("</tbody>");
  }

  /**
   * Add table footer for CALS model.
   */
  private void addCALSTableFooter(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateFooter()) {
      tableXMLFragment.append("<tfoot><row>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<entry></entry>");
      }
      tableXMLFragment.append("</row></tfoot>");
    }
  }

  /**
   * Add table header for CALS model.
   */
  private void addCALSTableHeader(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<thead><row>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<entry></entry>");
      }
      tableXMLFragment.append("</row></thead>");
    }
  }

  /**
   * Add column specification to the table XML fragment.
   */
  private void addCALSColSpecs(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    ColumnWidthsType columnsWidthsType = tableInfo.getColumnsWidthsType();
    String colWidth = null;
    if (columnsWidthsType == ColumnWidthsType.PROPORTIONAL_COL_WIDTHS) {
      // Proportional widths
      colWidth = "1.0*";
    } else if (columnsWidthsType == ColumnWidthsType.FIXED_COL_WIDTHS) {
      // Fixed widths
      colWidth = "75pt";
    }
    for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
      tableXMLFragment.append("<colspec colname=\"c" + i + "\" colnum=\"" + i + "\"");
      if (colWidth != null) {
        tableXMLFragment.append(" colwidth=\"" + colWidth + "\"");
      }
      tableXMLFragment.append("/>");
    }
  }

  /**
   * The operation will display a dialog for choosing table attributes.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  public ArgumentDescriptor[] getArguments() {
    return ARGUMENTS;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  public String getDescription() {
    return "Insert a docbook table";
  }

  /**
   * Adds to the given XML fragment the elements (caption, tbody, thead, etc.) 
   * specific to the HTML table model.
   *  
   * @param tableXMLFragment The string buffer where to append HTML elements.
   * @param tableInfo The table information.
   * @param namespace The table element namespace.
   */
  private void addHTMLXMLFragment(StringBuffer tableXMLFragment, TableInfo tableInfo, String namespace) {    
    if (tableInfo.getTitle() != null) {
      tableXMLFragment.append("<caption>" + tableInfo.getTitle() + "</caption>");
    }
    
    // Add table columns specification
    addHTMLTableCols(tableXMLFragment, tableInfo);
    
    // Add table header
    addHTMLTableHeader(tableXMLFragment, tableInfo);

    // Add table footer
    addHTMLTableFooter(tableXMLFragment, tableInfo);

    // Add table body
    addHTMLTableBody(tableXMLFragment, tableInfo);
  }
  
  /**
   * Add table columns specification.
   */
  private void addHTMLTableCols(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    ColumnWidthsType columnsWidthsType = tableInfo.getColumnsWidthsType();
    if (columnsWidthsType != ColumnWidthsType.DYNAMIC_COL_WIDTHS) {
      String colWidth = null;
      if (columnsWidthsType == ColumnWidthsType.PROPORTIONAL_COL_WIDTHS) {
        // Proportional widths
        colWidth = "1.0*";
      } else {
        // Fixed widths
        colWidth = "75pt";
      }
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<col width=\"" + colWidth + "\"/>");
      }
    }
  }

  /**
   * Add table header for HTML model.
   */
  private static void addHTMLTableHeader(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<thead><tr>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<th></th>");
      }
      tableXMLFragment.append("</tr></thead>");
    }
  }
  
  /**
   * Add table body for HTML model.
   */
  private void addHTMLTableBody(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    tableXMLFragment.append("<tbody>");
    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<tr>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        tableXMLFragment.append("<td></td>"); 
      }
      tableXMLFragment.append("</tr>"); 
    }
    tableXMLFragment.append("</tbody>");
  }

  /**
   * Add table footer for HTML model.
   */
  private static void addHTMLTableFooter(StringBuffer tableXMLFragment, TableInfo tableInfo) {
    if (tableInfo.isGenerateFooter()) {
      tableXMLFragment.append("<tfoot><tr>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        tableXMLFragment.append("<td></td>");
      }
      tableXMLFragment.append("</tr></tfoot>");
    }
  }
}