/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.xhtml;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentType;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.access.AuthorWorkspaceAccess;
import ro.sync.ecss.images.ImageHandlerDispatcher;
import ro.sync.ecss.images.xmlimages.XMLImageHandler;
import ro.sync.ecss.images.xmlimages.AbstractXMLImageHandler.CannotEditException;

/**
 * Operation used to insert an MathML Equation in XHTML 1.1 documents.
 */
public class InsertEquationOperation implements AuthorOperation {

  
  /**
   * The MathML fragment representing the default equation. In the XHTML 1.1 DTD it is not
   * prefixed.
   */
  private static String MATH_ML = 
      "<math xmlns='http://www.w3.org/1998/Math/MathML'><mrow>" + 
      "<msup><mi>a</mi><mn>2</mn></msup>" +
      "<mo>=</mo>" +
      "<msup><mi>b</mi><mn>2</mn></msup>" +
      "<mo>+</mo>" +
      "<msup><mi>c</mi><mn>2</mn>" + 
      "</msup></mrow></math>";

  /**
  * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
  */
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    try {
      AuthorWorkspaceAccess workspace = authorAccess.getWorkspaceAccess(); 
      if (!workspace.isStandalone()) {
        workspace.showInformationMessage("MathML editing is not supported on Eclipse.");
      } else {
        XMLImageHandler handler = ImageHandlerDispatcher.getInstance()
            .getXMLImageHandlerFor("mathml");
        if(handler != null) {
          String serializedDoctype = null;
          AuthorDocumentType dt = authorAccess.getDocumentController().getDoctype();
          if(dt != null){
            serializedDoctype = dt.getContent();
          }
          
          String mml = handler.editImage(MATH_ML,
              serializedDoctype,
              authorAccess.getEditorAccess().getEditorLocation().toString());
          
          if(mml != null){          
            int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
            
            // Inserts the equation
            authorAccess.getDocumentController().insertXMLFragmentSchemaAware(mml, caretOffset);
          }
        } else {
          //Maybe running from the Author component applet
          workspace.showInformationMessage("MathML editing is not supported.");
        }
      }
    } catch (CannotEditException e) {
      throw new AuthorOperationException(e.getMessage(), e);
    }
  }

  /**
  * No arguments. The operation will display a dialog for choosing the image fileref.
  * 
  * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
  */
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
  * @see ro.sync.ecss.extensions.api.Extension#getDescription()
  */
  public String getDescription() {
    return "Insert a MathML equation.";
  }
}